@@ -5,6 +5,7 @@ The :mod:`tslearn.clustering` module gathers time series specific clustering alg
 from __future__ import print_function
 from sklearn.base import BaseEstimator, ClusterMixin
 from sklearn.cluster.k_means_ import _k_init
+from sklearn.metrics.cluster import silhouette_score as sklearn_silhouette_score
 from sklearn.utils import check_random_state
 from scipy.spatial.distance import cdist
 import numpy
@@ -67,6 +68,75 @@ def _compute_inertia(distances, assignments, squared=True):
         return numpy.sum(distances[numpy.arange(n_ts), assignments]) / n_ts
 
 
+def silhouette_score(X, labels, metric=None, sample_size=None,
+                     random_state=None, **kwds):
+    """Compute the mean Silhouette Coefficient of all samples.
+
+    Read more in the `scikit-learn documentation
+    <http://scikit-learn.org/stable/modules/clustering.html#silhouette-coefficient>`_.
+
+    Parameters
+    ----------
+    X : array [n_ts, n_ts] if metric == "precomputed", or, \
+             [n_ts, sz, d] otherwise
+        Array of pairwise distances between time series, or a time series dataset.
+    labels : array, shape = [n_ts]
+         Predicted labels for each time series.
+    metric : string, or callable
+        The metric to use when calculating distance between time series.
+        Should be a metric from :ref:`tslearn.metrics <mod-metrics>` If X is the distance
+        array itself, use ``metric="precomputed"``.
+    sample_size : int or None
+        The size of the sample to use when computing the Silhouette Coefficient
+        on a random subset of the data.
+        If ``sample_size is None``, no sampling is used.
+    random_state : int, RandomState instance or None, optional (default=None)
+        The generator used to randomly select a subset of samples.  If int,
+        random_state is the seed used by the random number generator; If
+        RandomState instance, random_state is the random number generator; If
+        None, the random number generator is the RandomState instance used by
+        `np.random`. Used when ``sample_size is not None``.
+    **kwds : optional keyword parameters
+        Any further parameters are passed directly to the distance function.
+    Returns
+    -------
+    silhouette : float
+        Mean Silhouette Coefficient for all samples.
+    References
+    ----------
+    .. [1] `Peter J. Rousseeuw (1987). "Silhouettes: a Graphical Aid to the
+       Interpretation and Validation of Cluster Analysis". Computational
+       and Applied Mathematics 20: 53-65.
+       <http://www.sciencedirect.com/science/article/pii/0377042787901257>`_
+    .. [2] `Wikipedia entry on the Silhouette Coefficient
+           <https://en.wikipedia.org/wiki/Silhouette_(clustering)>`_
+    Examples
+    --------
+    >>> from tslearn.generators import random_walks
+    >>> from tslearn.metrics import soft_dtw
+    >>> X = random_walks(n_ts=50, sz=32, d=1)
+    >>> labels = numpy.random.randint(2, size=50)
+    >>> s_sc = silhouette_score(X, labels, metric=dtw)
+    >>> s_sc2 = silhouette_score(X, labels, metric=soft_dtw)
+    >>> s_sc3 = silhouette_score(cdist_dtw(X), labels, metric="precomputed")
+    """
+    if metric == "precomputed":
+        sklearn_X = X
+        sklearn_metric = metric
+    else:
+        X_ = to_time_series_dataset(X)
+        n, sz, d = X_.shape
+        sklearn_X = X_.reshape((n, -1))
+        if metric is None:
+            metric = dtw
+        sklearn_metric = lambda x, y: metric(x.reshape((sz, d)), y.reshape((sz, d)))
+    return sklearn_silhouette_score(X=sklearn_X,
+                                    labels=labels,
+                                    metric=sklearn_metric,
+                                    sample_size=sample_size,
+                                    random_state=random_state, **kwds)
+
+
 class GlobalAlignmentKernelKMeans(BaseEstimator, ClusterMixin):
     """Global Alignment Kernel K-means.
 
@@ -294,6 +364,8 @@ class TimeSeriesKMeans(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClust
     metric_params : dict or None
         Parameter values for the chosen metric. Value associated to the `"gamma_sdtw"` key corresponds to the gamma
         parameter in Soft-DTW.
+    dtw_inertia: bool
+        Whether to compute DTW inertia even if DTW is not the chosen metric.
     verbose : bool (default: True)
         Whether or not to print information about the inertia while learning the model.
     random_state : integer or numpy.RandomState, optional
@@ -309,6 +381,10 @@ class TimeSeriesKMeans(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClust
     inertia_ : float
         Sum of distances of samples to their closest cluster center.
 
+    Note
+    ----
+        If `metric` is set to `"euclidean"`, the algorithm expects a dataset of equal-sized time series.
+
     Examples
     --------
     >>> from tslearn.generators import random_walks
@@ -350,7 +426,7 @@ class TimeSeriesKMeans(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClust
     """
 
     def __init__(self, n_clusters=3, max_iter=50, tol=1e-6, n_init=1, metric="euclidean", max_iter_barycenter=100,
-                 metric_params=None, verbose=True, random_state=None):
+                 metric_params=None, dtw_inertia=False, verbose=True, random_state=None):
         self.n_clusters = n_clusters
         self.max_iter = max_iter
         self.tol = tol
@@ -360,6 +436,7 @@ class TimeSeriesKMeans(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClust
         self.verbose = verbose
         self.max_iter_barycenter = max_iter_barycenter
         self.max_attempts = max(self.n_init, 10)
+        self.dtw_inertia = dtw_inertia
 
         self.labels_ = None
         self.inertia_ = numpy.inf
@@ -369,9 +446,7 @@ class TimeSeriesKMeans(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClust
 
         if metric_params is None:
             metric_params = {}
-
-        if self.metric == "softdtw":
-            self.gamma_sdtw = metric_params.get("gamma_sdtw", 1.)
+        self.gamma_sdtw = metric_params.get("gamma_sdtw", 1.)
 
     def _fit_one_init(self, X, x_squared_norms, rs):
         n_ts, sz, d = X.shape
@@ -407,7 +482,11 @@ class TimeSeriesKMeans(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClust
         if update_class_attributes:
             self.labels_ = matched_labels
             _check_no_empty_cluster(self.labels_, self.n_clusters)
-            self.inertia_ = _compute_inertia(dists, self.labels_, self._squared_inertia)
+            if self.dtw_inertia and self.metric != "dtw":
+                inertia_dists = cdist_dtw(X, self.cluster_centers_)
+            else:
+                inertia_dists = dists
+            self.inertia_ = _compute_inertia(inertia_dists, self.labels_, self._squared_inertia)
         return matched_labels
 
     def _update_centroids(self, X):
@@ -525,6 +604,10 @@ class KShape(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClusteringMixin
     inertia_ : float
         Sum of distances of samples to their closest cluster center.
 
+    Note
+    ----
+        This method requires a dataset of equal-sized time series.
+
     Examples
     --------
     >>> from tslearn.generators import random_walks
@@ -561,10 +644,13 @@ class KShape(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClusteringMixin
         self.inertia_ = numpy.inf
         self.cluster_centers_ = None
 
+        self._norms = 0.
+        self._norms_centroids = 0.
+
     def _shape_extraction(self, X, k):
         sz = X.shape[1]
         Xp = y_shifted_sbd_vec(self.cluster_centers_[k], X[self.labels_ == k], norm_ref=-1,
-                               norms_dataset=numpy.array([-1.]))  # TODO: provide norms
+                               norms_dataset=self._norms[self.labels_ == k])
         S = numpy.dot(Xp[:, :, 0].T, Xp[:, :, 0])
         Q = numpy.eye(sz) - numpy.ones((sz, sz)) / sz
         M = numpy.dot(Q.T, numpy.dot(S, Q))
@@ -584,10 +670,11 @@ class KShape(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClusteringMixin
         for k in range(self.n_clusters):
             self.cluster_centers_[k] = self._shape_extraction(X, k)
         self.cluster_centers_ = TimeSeriesScalerMeanVariance(mu=0., std=1.).fit_transform(self.cluster_centers_)
+        self._norms_centroids = numpy.linalg.norm(self.cluster_centers_, axis=(1, 2))
 
     def _cross_dists(self, X):
-        return 1. - cdist_normalized_cc(X, self.cluster_centers_, norms1=numpy.array([-1.]), norms2=numpy.array([-1.]),
-                                        self_similarity=False)  # TODO: provide norms
+        return 1. - cdist_normalized_cc(X, self.cluster_centers_, norms1=self._norms,
+                                        norms2=self._norms_centroids, self_similarity=False)
 
     def _assign(self, X):
         dists = self._cross_dists(X)
@@ -599,6 +686,7 @@ class KShape(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClusteringMixin
         n_samples, sz, d = X.shape
         self.labels_ = rs.randint(self.n_clusters, size=n_samples)
         self.cluster_centers_ = rs.randn(self.n_clusters, sz, d)
+        self._norms_centroids = numpy.linalg.norm(self.cluster_centers_, axis=(1, 2))
         old_inertia = numpy.inf
 
         for it in range(self.max_iter):
@@ -631,6 +719,7 @@ class KShape(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClusteringMixin
         """
 
         X_ = to_time_series_dataset(X)
+        self._norms = numpy.linalg.norm(X_, axis=(1, 2))
         X_ = TimeSeriesScalerMeanVariance(mu=0., std=1.).fit_transform(X_)
         assert X_.shape[-1] == 1, "kShape is supposed to work on monomodal data, provided data has dimension %d" % \
                                   X_.shape[-1]
@@ -654,6 +743,7 @@ class KShape(BaseEstimator, ClusterMixin, TimeSeriesCentroidBasedClusteringMixin
                 if self.verbose:
                     print("Resumed because of empty cluster")
         self._post_fit(X_, best_correct_centroids, min_inertia)
+        self._norms_centroids = numpy.linalg.norm(self.cluster_centers_, axis=(1, 2))
         return self
 
     def fit_predict(self, X, y=None):
