@@ -12,7 +12,7 @@ from tslearn.cydtw import dtw as cydtw, dtw_path as cydtw_path, cdist_dtw as cyc
 from tslearn.cydtw import lb_envelope as cylb_envelope
 from tslearn.cydtw import sakoe_chiba_mask as cysakoe_chiba_mask, itakura_mask as cyitakura_mask
 from tslearn.cygak import cdist_gak as cycdist_gak, normalized_gak as cynormalized_gak
-from tslearn.utils import to_time_series, to_time_series_dataset
+from tslearn.utils import to_time_series, to_time_series_dataset, ts_size, check_equal_size
 
 __author__ = 'Romain Tavenard romain.tavenard[at]univ-rennes2.fr'
 
@@ -188,9 +188,9 @@ def cdist_dtw(dataset1, dataset2=None, global_constraint=None, sakoe_chiba_radiu
     >>> cdist_dtw([[1, 2, 2, 3], [1., 2., 3., 4.]])  # doctest: +NORMALIZE_WHITESPACE
     array([[ 0., 1.],
            [ 1., 0.]])
-    >>> cdist_dtw([[1, 2, 2, 3], [1., 2., 3., 4.]], [[1, 2, 3], [2, 3, 4]])  # doctest: +NORMALIZE_WHITESPACE
-    array([[ 0. ,  1.41421356],
-           [ 1. ,  1. ]])
+    >>> cdist_dtw([[1, 2, 2, 3], [1., 2., 3., 4.]], [[1, 2, 3], [2, 3, 4, 5]])  # doctest: +NORMALIZE_WHITESPACE +ELLIPSIS
+    array([[ 0. ,  2.449...],
+           [ 1. ,  1.414...]])
 
     See Also
     --------
@@ -255,7 +255,8 @@ def gak(s1, s2, sigma=1.):
     """
     s1 = to_time_series(s1)
     s2 = to_time_series(s2)
-    return cynormalized_gak(s1, s2, sigma)
+    gamma = 1. / (2 * sigma ** 2)
+    return numpy.exp(-soft_dtw(s1, s2, gamma=gamma) / gamma)  #cynormalized_gak(s1, s2, sigma)
 
 
 def cdist_gak(dataset1, dataset2=None, sigma=1.):
@@ -282,9 +283,9 @@ def cdist_gak(dataset1, dataset2=None, sigma=1.):
     >>> cdist_gak([[1, 2, 2, 3], [1., 2., 3., 4.]], sigma=2.)  # doctest: +NORMALIZE_WHITESPACE +ELLIPSIS
     array([[ 1. , 0.656...],
            [ 0.656..., 1. ]])
-    >>> cdist_gak([[1, 2, 2, 3], [1., 2., 3., 4.]], [[1, 2, 2, 3], [1., 2., 3., 4.]], sigma=2.)  # doctest: +NORMALIZE_WHITESPACE +ELLIPSIS
-    array([[ 1. , 0.656...],
-           [ 0.656..., 1. ]])
+    >>> cdist_gak([[1, 2, 2], [1., 2., 3., 4.]], [[1, 2, 2, 3], [1., 2., 3., 4.]], sigma=2.)  # doctest: +NORMALIZE_WHITESPACE +ELLIPSIS
+    array([[ 0.710...,  0.297...],
+           [ 0.656...,  1.        ]])
 
     See Also
     --------
@@ -301,7 +302,7 @@ def cdist_gak(dataset1, dataset2=None, sigma=1.):
         self_similarity = True
     else:
         dataset2 = to_time_series_dataset(dataset2)
-    return cycdist_gak(dataset1, dataset2, sigma, self_similarity=self_similarity)
+    return numpy.array([[gak(s1, s2, sigma=sigma) for s2 in dataset2] for s1 in dataset1])# cycdist_gak(dataset1, dataset2, sigma, self_similarity=self_similarity)
 
 
 def sigma_gak(dataset, n_samples=100, random_state=None):
@@ -342,12 +343,14 @@ def sigma_gak(dataset, n_samples=100, random_state=None):
     random_state = check_random_state(random_state)
     dataset = to_time_series_dataset(dataset)
     n_ts, sz, d = dataset.shape
+    if not check_equal_size(dataset):
+        sz = numpy.min([ts_size(ts) for ts in dataset])
     if n_ts * sz < n_samples:
         replace = True
     else:
         replace = False
     sample_indices = random_state.choice(n_ts * sz, size=n_samples, replace=replace)
-    dists = pdist(dataset.reshape((-1, d))[sample_indices], metric="euclidean")
+    dists = pdist(dataset[:, :sz, :].reshape((-1, d))[sample_indices], metric="euclidean")
     return numpy.median(dists) * numpy.sqrt(sz)
 
 
@@ -407,6 +410,11 @@ def lb_keogh(ts_query, ts_candidate=None, radius=1, envelope_candidate=None):
     envelope_candidate: pair of array-like (envelope_down, envelope_up) or None (default: None)
         Pre-computed envelope of the candidate time series. If set to None, it is computed based on `ts_candidate`.
 
+    Note
+    ----
+        This method requires a `ts_query` and `ts_candidate` (or `envelope_candidate`, depending on the call)
+        to be of equal size.
+
     Returns
     -------
     float
@@ -530,7 +538,7 @@ def soft_dtw(ts1, ts2, gamma=1.):
     """
     if gamma == 0.:
         return dtw(ts1, ts2)
-    return SoftDTW(SquaredEuclidean(ts1, ts2), gamma=numpy.float64(gamma)).compute()
+    return SoftDTW(SquaredEuclidean(ts1[:ts_size(ts1)], ts2[:ts_size(ts2)]), gamma=numpy.float64(gamma)).compute()
 
 
 def cdist_soft_dtw(dataset1, dataset2=None, gamma=1.):
@@ -577,12 +585,22 @@ def cdist_soft_dtw(dataset1, dataset2=None, gamma=1.):
     else:
         dataset2 = to_time_series_dataset(dataset2, dtype=numpy.float64)
     dists = numpy.empty((dataset1.shape[0], dataset2.shape[0]))
+    equal_size_ds1 = check_equal_size(dataset1)
+    equal_size_ds2 = check_equal_size(dataset2)
     for i, ts1 in enumerate(dataset1):
+        if equal_size_ds1:
+            ts1_short = ts1
+        else:
+            ts1_short = ts1[:ts_size(ts1)]
         for j, ts2 in enumerate(dataset2):
+            if equal_size_ds2:
+                ts2_short = ts2
+            else:
+                ts2_short = ts2[:ts_size(ts2)]
             if self_similarity and j < i:
                 dists[i, j] = dists[j, i]
             else:
-                dists[i, j] = soft_dtw(ts1, ts2, gamma=gamma)
+                dists[i, j] = soft_dtw(ts1_short, ts2_short, gamma=gamma)
 
     return dists
 
